import subprocess as sp
import time
import signal
import random
import os

LIBLITMUS = "~/liblitmus"
FEATHER_TRACE = "~/feather-trace-tools"

RUN_GLOBAL_PLANNER = True
RUN_LOCAL_PLANNER = True
RUN_LANE_DETECTION = True

LOCAL_PLANNER_APF = False

RUN_VEHICLE_TRACKING = True
RUN_PEDESTRIAN_TRACKING = True

NUM_TRACKING_FRAMES = 300
PARALLELISM = 2

OUTPUT_DIR = "detector_results_test"

def main():
    rrt_log_file = None
    apf_log_file = None
    pp_log_file = None
    lane_detection_log_file = None
    vehicle_tracking_log_file = None
    pedestrian_tracking_log_file = None

    if RUN_VEHICLE_TRACKING:
        t = sp.Popen(['mkdir', '-p', OUTPUT_DIR + "/vehicle/"])
        t.wait()

    if RUN_PEDESTRIAN_TRACKING:
        t = sp.Popen(['mkdir', '-p', OUTPUT_DIR + "/pedestrian/"])
        t.wait()

    if (not RUN_VEHICLE_TRACKING) and (not RUN_PEDESTRIAN_TRACKING):
        t = sp.Popen(['mkdir', OUTPUT_DIR])
        t.wait()

    taskset = []

    def cleanup_rtspin(*args):
        for task in taskset:
            task.kill()
        exit()

    signal.signal(signal.SIGINT, cleanup_rtspin)
    signal.signal(signal.SIGSEGV, cleanup_rtspin)
    signal.signal(signal.SIGTERM, cleanup_rtspin)

    trace = sp.Popen([LIBLITMUS + '/setsched GSN-EDF'], shell=True)
    trace.wait()
    time.sleep(2.0)

    trace = sp.Popen([FEATHER_TRACE + '/st-trace-schedule', 'temp-trace'], stdin=sp.PIPE)
    print('starting up tracing framework...')
    time.sleep(5.0)

    ########################################################
    # Motion Planning
    ########################################################

    taskset = []

    if RUN_GLOBAL_PLANNER:
        rrt_log_file = open(OUTPUT_DIR + "/rrt.log", 'w')
        print("Starting RRT")
        rrt = sp.Popen(["python3", "~/motion_planning/python_src/rrts/main_rrt2D.py"],
                       stdout=rrt_log_file)
        taskset.append(rrt)

        time.sleep(2)

    if RUN_LOCAL_PLANNER:
        if LOCAL_PLANNER_APF:
            apf_log_file = open(OUTPUT_DIR + "/apf.log", 'w')
            print("Starting APF")
            apf = sp.Popen(["python3", "~/motion_planning/python_src/adaptive_formation/gradient_interactive.py"],
                           stdout=apf_log_file)
            taskset.append(apf)
        else:
            pp_log_file = open(OUTPUT_DIR + "/pp.log", 'w')
            print("Starting Pure Pursuit")
            pp = sp.Popen(["python3", "~/pure_pursuit/pure_pursuit.py"],
                          stdout=pp_log_file)
            taskset.append(pp)

        time.sleep(2)

    ########################################################
    # Lane Detection
    ########################################################

    if RUN_LANE_DETECTION:
        lane_detection_log_file = open(OUTPUT_DIR + "/lane_detection.log", 'w')
        print("Starting lane detection")
        # Need at least 10*300 jobs
        lane_detection = sp.Popen(["~/opencv/build/bin/example_rt_gpu_lane_detection",
                                "--folder",
                                "rgb_lane_detection/",
                                "--display", "false",
                                "--count", "5000",
                                "--rt_period", "200",
                                "--rt_deadline", "200"],
                                stdout=lane_detection_log_file)
        taskset.append(lane_detection)

        time.sleep(15)

    ########################################################
    # Vehicle Tracking
    ########################################################

    vehicle_tracking_tasks = []

    if RUN_VEHICLE_TRACKING:
        print("Starting vehicle tracking")
        for i in range(PARALLELISM):
            vehicle_tracking_i = sp.Popen(["~/carla-car-detection/env/bin/python",
                                        "~/carla-car-detection/image_detection.py",
                                        "--model_path=~/carla-car-detection/full_trained_model/detectors-42082/",
                                        "--images_path=rgb_tracking/",
                                        "--min_threshold=0.70",
                                        "--output_path={0}".format(OUTPUT_DIR + "/vehicle/"),
                                        '--class_list=vehicle,bike,motobike',
                                        '--class_str=vehicle',
                                        "--parallelism={0}".format(PARALLELISM),
                                        "--index={0}".format(i),
                                        "--num_files={0}".format(NUM_TRACKING_FRAMES)])
            vehicle_tracking_tasks.append(vehicle_tracking_i)
        taskset.extend(vehicle_tracking_tasks)

        time.sleep(40)

    ########################################################
    # Pedestrian Tracking
    ########################################################

    pedestrian_tracking_tasks = []

    if RUN_PEDESTRIAN_TRACKING:
        print("Starting pedestrian tracking")
        for i in range(PARALLELISM):
            pedestrian_tracking_i = sp.Popen(["~/carla-car-detection/env/bin/python",
                                        "~/carla-car-detection/image_detection.py",
                                        "--model_path=~/carla-car-detection/full_trained_model/detectors-42082/",
                                        "--images_path=rgb_tracking/",
                                        "--min_threshold=0.70",
                                        "--output_path={0}".format(OUTPUT_DIR + "/pedestrian/"),
                                        '--class_list=pedestrian',
                                        '--class_str=pedestrian',
                                        "--parallelism={0}".format(PARALLELISM),
                                        "--index={0}".format(i),
                                        "--num_files={0}".format(NUM_TRACKING_FRAMES)])
            pedestrian_tracking_tasks.append(pedestrian_tracking_i)
        taskset.extend(pedestrian_tracking_tasks)

        time.sleep(40)

    ########################################################
    # Release tasks, then kill them all after some time
    ########################################################

    releaser = sp.Popen([LIBLITMUS + '/release_ts'])
    releaser.wait()

    # # Wait for 60 seconds
    # time.sleep(120)
    # print('specified time has passed')
    # print('letting system settle down')

    # Wait until the tracking tasks finish
    if RUN_PEDESTRIAN_TRACKING or RUN_VEHICLE_TRACKING:
        for task in vehicle_tracking_tasks + pedestrian_tracking_tasks:
            task.wait()
    else:
        time.sleep(20)

    # After waiting, all tasks are killed
    for task in taskset:
        task.kill()

    for task in taskset:
        task.wait()

    if rrt_log_file != None:
        rrt_log_file.close()
    if apf_log_file != None:
        apf_log_file.close()
    if pp_log_file != None:
        pp_log_file.close()
    if lane_detection_log_file != None:
        lane_detection_log_file.close()

    time.sleep(300)

    print('ending tracing')
    trace.communicate(b'\n')
    trace.wait()

    # Set the scheduler back to Linux
    trace = sp.Popen([LIBLITMUS + '/setsched', 'Linux'], shell=False)
    trace.wait()

if __name__ == '__main__':
    main()
